/* Scrutari - Copyright (c) 2008-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.htmlpages;

import fr.exemole.sctserver.api.ScrutariSession;
import fr.exemole.sctserver.api.SctEngine;
import fr.exemole.sctserver.servlets.admin.AdminConstants;
import fr.exemole.sctserver.servlets.admin.AdminResponseHandlerFactory;
import fr.exemole.sctserver.tools.EngineUtils;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.html.HtmlPrinterUtils;
import net.mapeadores.util.html.PageHtmlPrinter;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.servlets.ResponseHandler;


/**
 *
 * @author Vincent Calame
 */
public abstract class AdminHtmlPage extends PageHtmlPrinter implements ResponseHandler {

    private final static HTMLAttributes ERROR_ATTR = HA.classes("scrutari-admin-ErrorMessage");
    private final static HTMLAttributes DONE_ATTR = HA.classes("scrutari-admin-DoneMessage");
    private final static HTMLAttributes TOOLDIV_ATTR = HA.classes("scrutari-admin-Tool");
    private final static HTMLAttributes CURRENT_ATTR = HA.classes("scrutari-admin-Current");
    protected SctEngine engine;
    protected ScrutariSession scrutariSession;
    protected Lang langUi;
    private CommandMessage commandMessage;
    private short currentPage;

    public AdminHtmlPage() {
        addCssUrl("../css/admin.css");
    }

    protected void init(SctEngine engine) {
        this.engine = engine;
        this.scrutariSession = engine.getScrutariSession();
        setTitle(engine.getEngineName() + " – Scrutari");
    }

    public void setLangUi(Lang langUi) {
        this.langUi = langUi;
        addMessageLocalisation(EngineUtils.getMessageLocalisation(engine, langUi));
    }

    public void setCurrentPage(short currentPage) {
        this.currentPage = currentPage;
    }

    public void setCommandMessage(CommandMessage commandMessage) {
        this.commandMessage = commandMessage;
    }

    @Override
    public long getLastModified() {
        return ResponseHandler.LASTMODIFIED_NOCACHE;
    }

    protected void initResponse(HttpServletResponse response) throws IOException {
        response.setContentType("text/html;charset=UTF-8");
        initPrinter(response.getWriter());

    }

    protected void printTools() {
        startDiv(TOOLDIV_ATTR);
        printToolLink(AdminConstants.INDEX_PAGE);
        printToolSeparator();
        printToolLink(AdminConstants.STATS_PAGE);
        printToolSeparator();
        printToolLink(AdminConstants.FICHES_PAGE);
        printToolSeparator();
        printToolLink(AdminConstants.MOTSCLES_PAGE);
        printToolSeparator();
        printToolLink(AdminConstants.LEXIES_PAGE);
        printToolSeparator();
        printToolLink(AdminConstants.JOURNAUX_PAGE);
        endDiv();

    }

    protected void printMessage() {
        if (commandMessage != null) {
            HtmlPrinterUtils.printCommandMessage(this, commandMessage, DONE_ATTR, ERROR_ATTR);
        }
    }

    protected void start() {
        start(langUi);
    }

    private void printToolSeparator() {
        append(" — ");
    }

    private void printToolLink(short page) {
        String key = AdminResponseHandlerFactory.pageToString(page);
        if (currentPage == page) {
            startSpan(CURRENT_ATTR);
        } else {
            startA(HA.href(key + ".html"));
        }
        localize(AdminResponseHandlerFactory.getPageMessageKey(page));
        if (currentPage == page) {
            endSpan();
        } else {
            endA();
        }
    }

    public SctEngine getEngine() {
        return engine;
    }

}
