/* Scrutari - Copyright (c) 2008-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.htmlpages;

import fr.exemole.sctserver.api.SctEngine;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.html.HtmlPrinterUtils;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.ScrutariDB;
import net.scrutari.db.api.stats.CountStats;
import net.scrutari.db.tools.util.DataLibelleUtils;


/**
 *
 * @author Vincent Calame
 */
public class StatsPage extends AdminHtmlPage {

    private final ScrutariDB scrutariDB;

    public StatsPage(SctEngine engine) {
        init(engine);
        scrutariDB = scrutariSession.getScrutariDB();
        System.gc();
    }

    @Override
    public void handleResponse(HttpServletResponse response) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        initResponse(response);
        start();
        printTools();
        printMessage();
        CountStats countStats = scrutariDB.getStats().getEngineCountStats();
        startH2(null);
        localize("_ title.stats.stats");
        endH2();
        startTable();
        printRow("_ label.stats.corpus", countStats.getCorpusCount());
        printRow("_ label.stats.thesaurus", countStats.getThesaurusCount());
        printRow("_ label.stats.fiches", countStats.getFicheCount());
        printRow("_ label.stats.motscles", countStats.getMotcleCount());
        printRow("_ label.stats.lexie", countStats.getLexieCount());
        printRow("_ label.stats.indexation", countStats.getIndexationCount());
        endTable();
        startH2(null);
        localize("_ title.stats.memory");
        endH2();
        Runtime run = Runtime.getRuntime();
        startTable();
        printRow("_ label.stats.memory_free", (int) run.freeMemory() / (1024 * 1024));
        printRow("_ label.stats.memory_total", (int) run.totalMemory() / (1024 * 1024));
        printRow("_ label.stats.memory_max", (int) run.maxMemory() / (1024 * 1024));
        endTable();
        printCorpusURIList(dataAccess);
        printThesaurusURIList(dataAccess);
        end();
        dataAccess.close();
    }

    private void printRow(String key, int count) {
        startRow();
        startCell();
        HtmlPrinterUtils.localizeWithColon(this, key);
        endCell();
        startCell();
        append(String.valueOf(count));
        endCell();
        endRow();
    }

    private void printCorpusURIList(DataAccess dataAccess) {
        startH2(null);
        localize("_ title.stats.list_corpus");
        endH2();
        int baseCount = dataAccess.getBaseCount();
        startUl();
        for (int i = 0; i < baseCount; i++) {
            BaseData baseData = dataAccess.getBaseDataByIndex(i);
            printCorpusByBase(baseData, dataAccess);
        }
        endUl();
    }

    private void printThesaurusURIList(DataAccess dataAccess) {
        startH2(null);
        localize("_ title.stats.list_thesaurus");
        endH2();
        int baseCount = dataAccess.getBaseCount();
        startUl();
        for (int i = 0; i < baseCount; i++) {
            BaseData baseData = dataAccess.getBaseDataByIndex(i);
            printThesaurusByBase(baseData, dataAccess);
        }
        endUl();
    }

    private void printCorpusByBase(BaseData baseData, DataAccess dataAccess) {
        int corpusCount = baseData.getCorpusCount();
        startLi();
        startP();
        escape(DataLibelleUtils.checkLibelleString(baseData, BaseData.INTITULE_LONG, langUi, "?"));
        endP();
        startUl();
        for (int i = 0; i < corpusCount; i++) {
            CorpusData corpusData = dataAccess.getCorpusDataByCode(baseData.getCodeCorpus(i));
            printCorpusData(corpusData);
        }
        endUl();
        endLi();
    }

    private void printThesaurusByBase(BaseData baseData, DataAccess dataAccess) {
        int thesaurusCount = baseData.getThesaurusCount();
        startLi();
        startP();
        escape(DataLibelleUtils.checkLibelleString(baseData, BaseData.INTITULE_LONG, langUi, "?"));
        endP();
        startUl();
        for (int i = 0; i < thesaurusCount; i++) {
            ThesaurusData thesaurusData = dataAccess.getThesaurusDataByCode(baseData.getCodeThesaurus(i));
            printThesaurusData(thesaurusData);
        }
        endUl();
        endLi();
    }

    private void printCorpusData(CorpusData corpusData) {
        startLi();
        startP();
        escape(DataLibelleUtils.checkLibelleString(corpusData, CorpusData.INTITULECORPUS_FIELDNUMBER, langUi, "?"));
        append(" ");
        startCode();
        append("[");
        append(corpusData.getCorpusURI().toString());
        append("]");
        endCode();
        endP();
        endLi();
    }

    private void printThesaurusData(ThesaurusData thesaurusData) {
        startLi();
        startP();
        escape(DataLibelleUtils.checkLibelleString(thesaurusData, langUi, "?"));
        append(" ");
        startCode();
        append("[");
        append(thesaurusData.getThesaurusURI().toString());
        append("]");
        endCode();
        endP();
        endLi();
    }

}
