/* ScrutariLib_SearchEngine - Copyright (c) 2008-2015 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.lexieseek;

import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.SubstringPosition;
import net.mapeadores.util.text.collation.CollationUnitPosition;
import net.mapeadores.util.text.collation.map.SearchResultUnit;
import net.mapeadores.util.text.collation.map.SearchResultUnitList;
import net.mapeadores.util.text.search.SimpleSearchToken;
import net.scrutari.db.api.LexieAccess;
import net.scrutari.lexie.LexieOccurrences;
import net.scrutari.lexie.LexieOccurrencesList;
import net.scrutari.searchengine.api.operands.search.AlineaFilter;
import net.scrutari.searchengine.api.options.ComputedOptions;
import net.scrutari.searchengine.api.lexieseek.LexieSeekHandler;


/**
 *
 * @author Vincent Calame
 */
class SimpleTokenSeeker {

    private SimpleTokenSeeker() {
    }

    static void seek(SimpleSearchToken searchToken, int operandNumber, Lang lang, LexieSeeker lexieSeeker, AlineaFilter alineaFilter) {
        SearchResultUnitList<Long> list = lexieSeeker.search(searchToken.getTokenString(), lang, searchToken.getSearchType());
        if (list != null) {
            int count = list.getSearchResultUnitCount();
            for (int j = 0; j < count; j++) {
                SearchResultUnit<Long> searchResultItem = list.getSearchResultUnit(j);
                add(searchResultItem.getValue(), searchResultItem.getSearchTextPositionInCollatedKey(), operandNumber, lang, lexieSeeker, alineaFilter);
            }
        }
    }

    private static void add(long lexieId, SubstringPosition searchTextePositionInCollatedKey, int operandNumber, Lang lang, LexieSeeker lexieSeeker, AlineaFilter alineaFilter) {
        LexieAccess lexieAccess = lexieSeeker.getLexieAccess();
        ComputedOptions computedOptions = lexieSeeker.getComputedOptions();
        LexieSeekHandler lexieSeekHandler = lexieSeeker.getLexieSeekHandler();
        int[] codecorpusArray = computedOptions.getCodecorpusArray();
        if (codecorpusArray != null) {
            int corpusCount = codecorpusArray.length;
            for (int i = 0; i < corpusCount; i++) {
                int codecorpus = codecorpusArray[i];
                LexieOccurrencesList lexieOccurrencesList = lexieAccess.getLexieOccurrencesList(lexieId, codecorpus);
                int lexieCount = lexieOccurrencesList.getLexieOccurrencesCount();
                for (int j = 0; j < lexieCount; j++) {
                    LexieOccurrences lexieOccurrences = lexieOccurrencesList.getLexieOccurrences(j);
                    int codefiche = lexieOccurrences.getCode();
                    if (computedOptions.acceptFiche(codefiche)) {
                        int alineaCount = lexieOccurrences.getAlineaCount();
                        for (int k = 0; k < alineaCount; k++) {
                            int alineaInteger = lexieOccurrences.getAlineaInteger(k);
                            if (alineaFilter.acceptAlinea(codecorpus, alineaInteger)) {
                                CollationUnitPosition bestCollationUnitPosition = lexieOccurrences.getBestPositionInAlinea(k);
                                int bestAlineaInteger = alineaInteger;
                                SubstringPosition sourcePosition = bestCollationUnitPosition.convertToPositionInWholeSource(searchTextePositionInCollatedKey);
                                lexieSeekHandler.addAlineaOccurrence(codefiche, codecorpus, operandNumber, bestAlineaInteger, sourcePosition);
                                lexieSeeker.addFiche(codefiche, codecorpus);
                            }
                        }
                    }
                }
            }
        }
        int[] codethesaurusArray = computedOptions.getCodethesaurusArray();
        if (codethesaurusArray != null) {
            int thesaurusCount = codethesaurusArray.length;
            for (int j = 0; j < thesaurusCount; j++) {
                int codethesaurus = codethesaurusArray[j];
                if (!alineaFilter.acceptMotcle(codethesaurus)) {
                    continue;
                }
                LexieOccurrencesList lexieOccurrencesList = lexieAccess.getLexieOccurrencesList(lexieId, codethesaurus);
                int lexieCount = lexieOccurrencesList.getLexieOccurrencesCount();
                for (int k = 0; k < lexieCount; k++) {
                    LexieOccurrences p = lexieOccurrencesList.getLexieOccurrences(k);
                    int codemotcle = p.getCode();
                    if (computedOptions.acceptMotcle(codemotcle)) {
                        CollationUnitPosition bestCollationUnitPosition = p.getBestPositionInAlinea(0);
                        SubstringPosition sourcePosition = bestCollationUnitPosition.convertToPositionInWholeSource(searchTextePositionInCollatedKey);
                        float coverRate = ((float) sourcePosition.getLength()) / (p.getAlineaWholeLength(0));
                        lexieSeekHandler.addLangOccurrence(codemotcle, codethesaurus, operandNumber, lang, sourcePosition, coverRate);
                        lexieSeeker.addMotcle(codemotcle, codethesaurus);
                    }
                }
            }
        }
    }

}
