/* ScrutariLib_SearchEngine - Copyright (c) 2010-2015 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.options;

import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LangList;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.ScrutariDB;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.db.api.stats.FicheStats;
import net.scrutari.db.api.stats.ScrutariDBStats;
import net.scrutari.db.tools.util.DataUtils;
import net.scrutari.searchengine.api.operands.search.SearchOperation;
import net.scrutari.searchengine.api.options.ComputedOptions;
import net.scrutari.searchengine.api.options.SearchOptions;
import net.scrutari.searchengine.api.options.SearchOptionsDef;
import net.scrutari.searchengine.tools.EligibilityBuffer;
import net.scrutari.searchengine.tools.operands.search.SearchOperationUtils;


/**
 *
 * @author Vincent Calame
 */
public final class ComputedOptionsEngine {

    private ComputedOptionsEngine() {
    }

    public static ComputedOptions compute(ScrutariDB scrutariDB, DataAccess dataAccess, SearchOperation searchOperation, SearchOptions searchOptions, boolean withFiches, boolean withMotscles) {
        ComputedOptionsBuilder builder = new ComputedOptionsBuilder(searchOptions);
        boolean mandatoryOperand = SearchOperationUtils.hasMandatoryOperand(searchOperation);
        builder.setMandatoryOperand(mandatoryOperand);
        CodeList corpusCodeList = null;
        CodeList thesaurusCodeList = null;
        Lang[] langArray = null;
        if (searchOptions != null) {
            SearchOptionsDef searchOptionsDef = searchOptions.getSearchOptionsDef();
            LangList langList = searchOptionsDef.getFilterLangList();
            int langCount = langList.getLangCount();
            if (langCount > 0) {
                langArray = new Lang[langCount];
                for (int i = 0; i < langCount; i++) {
                    langArray[i] = langList.getLang(i);
                }
            }
            corpusCodeList = searchOptions.getCorpusCodeList();
            thesaurusCodeList = searchOptions.getThesaurusCodeList();
            EligibilityBuffer eligibilityBuffer = new EligibilityBuffer(dataAccess, searchOptionsDef);
            if ((langArray != null) && (!mandatoryOperand)) {
                eligibilityBuffer.addLangCheck(langArray);
            }
            builder.setEligibilityBuffer(eligibilityBuffer);
        }
        if (withFiches) {
            if (corpusCodeList == null) {
                corpusCodeList = dataAccess.getCorpusCodeList();
            }
            int corpusCount = corpusCodeList.getCodeCount();
            int[] codecorpusArray = new int[corpusCount];
            for (int i = 0; i < corpusCount; i++) {
                codecorpusArray[i] = corpusCodeList.getCode(i);
            }
            builder.setCodecorpusArray(codecorpusArray);
            builder.setFicheMaximum(DataUtils.getFicheCountByCorpus(dataAccess, codecorpusArray, langArray, scrutariDB.getStats()));
        }
        if (withMotscles) {
            if (thesaurusCodeList == null) {
                thesaurusCodeList = dataAccess.getThesaurusCodeList();
            }
            int thesaurusCount = thesaurusCodeList.getCodeCount();
            int[] codethesaurusArray = new int[thesaurusCount];
            for (int i = 0; i < thesaurusCount; i++) {
                codethesaurusArray[i] = thesaurusCodeList.getCode(i);
            }
            builder.setCodethesaurusArray(codethesaurusArray);
        }
        if (langArray == null) {
            builder.setLangArray(true, getAllLangArray(scrutariDB.getStats()));
        } else {
            builder.setLangArray(false, langArray);
        }
        CodeList ficheCodeList = searchOptions.getFicheCodeList();
        if (ficheCodeList != null) {
            int ficheCount = ficheCodeList.getCodeCount();
            for (int i = 0; i < ficheCount; i++) {
                builder.addCodefiche(ficheCodeList.getCode(i));
            }
        }
        return builder.toComputedOptions();
    }

    private static Lang[] getAllLangArray(ScrutariDBStats scrutariDBStats) {
        FicheStats ficheStats = scrutariDBStats.getEngineFicheStats();
        int count = ficheStats.getLangCount();
        Lang[] result = new Lang[count];
        for (int i = 0; i < count; i++) {
            result[i] = ficheStats.getLang(i);
        }
        return result;
    }

}
