/* ScrutariLib_SearchEngine - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.pertinence;

import net.scrutari.searchengine.api.pertinence.FichePertinence;


/**
 *
 * @author Vincent Calame
 */
public final class PertinencePack implements Comparable<PertinencePack> {

    private float globalScore;
    private float[] scoreArray;
    private FichePertinence fichePertinence;
    private int dateInteger;

    public PertinencePack(float[] scoreArray, FichePertinence fichePertinence, int dateInteger) {
        this.scoreArray = scoreArray;
        this.fichePertinence = fichePertinence;
        this.dateInteger = dateInteger;
        initGlobalScore(scoreArray);
    }

    private void initGlobalScore(float[] scoreArray) {
        this.globalScore = scoreArray[0] + scoreArray[1] + scoreArray[2];
    }

    public FichePertinence getFichePertinence() {
        return fichePertinence;
    }

    public float getPertinenceScore() {
        return globalScore;
    }

    public int[] getScoreArray() {
        int[] result = new int[3];
        result[0] = round(scoreArray[0]);
        result[1] = round(scoreArray[1]);
        result[2] = round(scoreArray[2]);
        return result;
    }

    @Override
    public boolean equals(Object other) {
        if (other == null) {
            return false;
        }
        if (other == this) {
            return true;
        }
        if (other.getClass() != this.getClass()) {
            return false;
        }
        PertinencePack otherPertinencePack = (PertinencePack) other;
        if (otherPertinencePack.globalScore != this.globalScore) {
            return false;
        }
        if (otherPertinencePack.dateInteger != this.dateInteger) {
            return false;
        }
        return (otherPertinencePack.fichePertinence.getCodefiche() == this.fichePertinence.getCodefiche());
    }

    @Override
    public int hashCode() {
        return (int) globalScore * 10000000 + fichePertinence.getCodefiche();
    }

    /**
     * Une instance de PertinencePack est plus petite qu'une autre quand sa pertinence
     * est meilleure !
     */
    public int compareTo(PertinencePack otherPertinencePack) {
        if (otherPertinencePack.globalScore > this.globalScore) {
            return 1;
        }
        if (otherPertinencePack.globalScore < this.globalScore) {
            return -1;
        }
        if (otherPertinencePack.dateInteger > this.dateInteger) {
            return 1;
        }
        if (otherPertinencePack.dateInteger < this.dateInteger) {
            return -1;
        }
        if (otherPertinencePack.fichePertinence.getCodefiche() > this.fichePertinence.getCodefiche()) {
            return 1;
        }
        if (otherPertinencePack.fichePertinence.getCodefiche() < this.fichePertinence.getCodefiche()) {
            return -1;
        }
        return 0;
    }

    public PertinencePack derive(float originScore) {
        float[] newScore = new float[3];
        newScore[0] = scoreArray[0];
        newScore[1] = scoreArray[1];
        newScore[2] = originScore;
        return new PertinencePack(newScore, fichePertinence, dateInteger);
    }

    private static int round(float f) {
        return (Math.round(f * 1000));
    }

}
