/* ScrutariLib_SearchEngine - Copyright (c) 2011-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.result;

import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.db.tools.codes.CodesUtils;
import net.scrutari.searchengine.api.result.FicheSearchResult;
import net.scrutari.searchengine.api.result.FicheSearchResultGroup;
import net.scrutari.searchengine.api.result.FicheSearchSource;
import net.scrutari.searchengine.api.result.MotcleSearchResultInfo;


/**
 *
 * @author Vincent Calame
 */
public class FicheSearchResultBuilder {

    private final InternalFicheSearchResult ficheSearchResult;
    private final Set<Integer> corpusSet = new LinkedHashSet<Integer>();
    private final ArrayList<MotcleSearchResultInfo> motcleList = new ArrayList<MotcleSearchResultInfo>();
    private final List<FicheSearchResultGroup> groupList = new ArrayList<FicheSearchResultGroup>();

    public FicheSearchResultBuilder(FicheSearchSource ficheSearchSource, boolean newSearch) {
        this.ficheSearchResult = new InternalFicheSearchResult(ficheSearchSource, newSearch);
    }

    public void setFicheMaximum(int ficheMaximum) {
        ficheSearchResult.ficheMaximum = ficheMaximum;
    }

    public void setFicheCount(int ficheCount) {
        ficheSearchResult.ficheCount = ficheCount;
    }

    public void setMotcleCount(int motcleCount) {
        motcleList.ensureCapacity(motcleCount);
        ficheSearchResult.motcleCount = motcleCount;
    }

    public void addCodecorpus(int codecorpus) {
        corpusSet.add(codecorpus);
    }

    public void addFicheSearchResultGroup(FicheSearchResultGroup ficheSearchResultGroup) {
        groupList.add(ficheSearchResultGroup);
    }

    public void addMotcleSearchResultInfo(MotcleSearchResultInfo motcleSearchResultInfo) {
        motcleList.add(motcleSearchResultInfo);
    }

    public FicheSearchResult toFicheSearchResult() {
        int groupSize = groupList.size();
        ficheSearchResult.ficheSearchResultGroupArray = groupList.toArray(new FicheSearchResultGroup[groupSize]);
        int motcleSize = motcleList.size();
        ficheSearchResult.motcleSearchResultInfoArray = motcleList.toArray(new MotcleSearchResultInfo[motcleSize]);
        ficheSearchResult.corpusCodeList = CodesUtils.toCodeList(corpusSet);
        return ficheSearchResult;
    }


    private static class InternalFicheSearchResult implements FicheSearchResult {

        private final FicheSearchSource ficheSearchSource;
        private final boolean newSearch;
        private int ficheCount;
        private int ficheMaximum;
        private int motcleCount;
        private FicheSearchResultGroup[] ficheSearchResultGroupArray;
        private MotcleSearchResultInfo[] motcleSearchResultInfoArray;
        private CodeList corpusCodeList;

        private InternalFicheSearchResult(FicheSearchSource ficheSearchSource, boolean newSearch) {
            this.ficheSearchSource = ficheSearchSource;
            this.newSearch = newSearch;
        }

        @Override
        public FicheSearchSource getFicheSearchSource() {
            return ficheSearchSource;
        }

        @Override
        public boolean isNewSearch() {
            return newSearch;
        }

        @Override
        public int getFicheCount() {
            return ficheCount;
        }

        @Override
        public int getFicheMaximum() {
            return ficheMaximum;
        }

        @Override
        public int getMotcleCount() {
            return motcleCount;
        }

        @Override
        public CodeList getCodecorpusList() {
            return corpusCodeList;
        }

        @Override
        public int getFicheSearchResultGroupCount() {
            if (ficheSearchResultGroupArray == null) {
                return 0;
            }
            return ficheSearchResultGroupArray.length;
        }

        @Override
        public FicheSearchResultGroup getFicheSearchResultGroup(int i) {
            return ficheSearchResultGroupArray[i];
        }

        @Override
        public MotcleSearchResultInfo getMotcleResult(int i) {
            return motcleSearchResultInfoArray[i];
        }

    }

}
