/* SctServer - Copyright (c) 2011-2015 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request;

import fr.exemole.sctserver.json.FicheDataJson;
import java.text.ParseException;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class FicheFieldsFilter {

    public final static short NONE_FILTER = 0;
    public final static short CONCERNED_FILTER = 1;
    public final static short ALL_FILTER = 2;
    public final static short PRIMARY_FILTER = 3;
    private final Map<AttributeKey, String> attributeKeyMap = new HashMap<AttributeKey, String>();
    private String titrePropertyName = null;
    private String soustitrePropertyName = null;
    private String hrefPropertyName = null;
    private String anneePropertyName = null;
    private boolean withCodeCorpus = false;
    private boolean withCodeBase = false;
    private boolean withFicheId = false;
    private boolean withLang = false;
    private boolean withDate = false;
    private boolean withFicheIcon = false;
    private boolean withGeo = false;
    private boolean withMTitre = false;
    private boolean withMSoustitre = false;
    private boolean withComplements = false;
    private boolean withMComplements = false;
    private boolean allMComplements = false;
    private boolean withScore = false;
    private boolean withCorpusName = false;
    private boolean withBaseName = false;
    private boolean withIcon = false;
    private short mAttributesFilter = NONE_FILTER;

    public FicheFieldsFilter() {
    }

    public boolean isWithCodeCorpus() {
        return withCodeCorpus;
    }

    public void setWithCodeCorpus(boolean withCodeCorpus) {
        this.withCodeCorpus = withCodeCorpus;
    }

    public boolean isWithCodeBase() {
        return withCodeBase;
    }

    public void setWithCodeBase(boolean withCodeBase) {
        this.withCodeBase = withCodeBase;
    }

    public String getTitrePropertyName() {
        return titrePropertyName;
    }

    public boolean setTitrePropertyName(String titrePropertyName) {
        if (titrePropertyName == null) {
            this.titrePropertyName = null;
            return true;
        }
        if (titrePropertyName.equals("titre")) {
            this.titrePropertyName = "titre";
            return true;
        }
        if (!isValidName(titrePropertyName)) {
            return false;
        }
        this.titrePropertyName = titrePropertyName;
        return true;
    }

    public String getSoustitrePropertyName() {
        return soustitrePropertyName;
    }

    public boolean setSoustitrePropertyName(String soustitrePropertyName) {
        if (soustitrePropertyName == null) {
            this.soustitrePropertyName = null;
            return true;
        }
        if (soustitrePropertyName.equals("soustitre")) {
            this.soustitrePropertyName = "soustitre";
            return true;
        }
        if (!isValidName(soustitrePropertyName)) {
            return false;
        }
        this.soustitrePropertyName = soustitrePropertyName;
        return true;
    }

    public String getHrefPropertyName() {
        return hrefPropertyName;
    }

    public boolean setHrefPropertyName(String hrefPropertyName) {
        if (hrefPropertyName == null) {
            this.hrefPropertyName = null;
            return true;
        }
        if (hrefPropertyName.equals("href")) {
            this.hrefPropertyName = "href";
            return true;
        }
        if (!isValidName(hrefPropertyName)) {
            return false;
        }
        this.hrefPropertyName = hrefPropertyName;
        return true;
    }

    public String getAnneePropertyName() {
        return anneePropertyName;
    }

    public boolean setAnneePropertyName(String anneePropertyName) {
        if (anneePropertyName == null) {
            this.anneePropertyName = null;
            return true;
        }
        if (anneePropertyName.equals("annee")) {
            this.anneePropertyName = "annee";
            return true;
        }
        if (!isValidName(anneePropertyName)) {
            return false;
        }
        this.anneePropertyName = anneePropertyName;
        return true;
    }

    public boolean isWithFicheId() {
        return withFicheId;
    }

    public void setWithFicheId(boolean withFicheId) {
        this.withFicheId = withFicheId;
    }

    public boolean isWithLang() {
        return withLang;
    }

    public void setWithLang(boolean withLang) {
        this.withLang = withLang;
    }

    public boolean isWithDate() {
        return withDate;
    }

    public void setWithDate(boolean withDate) {
        this.withDate = withDate;
    }

    public boolean isWithFicheIcon() {
        return withFicheIcon;
    }

    public void setWithFicheIcon(boolean withFicheIcon) {
        this.withFicheIcon = withFicheIcon;
    }

    public boolean isWithGeo() {
        return withGeo;
    }

    public void setWithGeo(boolean withGeo) {
        this.withGeo = withGeo;
    }

    public boolean isWithMTitre() {
        return withMTitre;
    }

    public void setWithMTitre(boolean withMTitre) {
        this.withMTitre = withMTitre;
    }

    public boolean isWithMSoustitre() {
        return withMSoustitre;
    }

    public void setWithMSoustitre(boolean withMSoustitre) {
        this.withMSoustitre = withMSoustitre;
    }

    public boolean isWithComplements() {
        return withComplements;
    }

    public void setWithComplements(boolean withComplements) {
        this.withComplements = withComplements;
    }

    public boolean isWithMComplements() {
        return withMComplements;
    }

    public void setWithMComplements(boolean withMComplements) {
        this.withMComplements = withMComplements;
    }

    public boolean isAllMComplements() {
        return allMComplements;
    }

    public void setAllMComplements(boolean withComplementsDefault) {
        this.allMComplements = withComplementsDefault;
    }

    public short getMAttributesFilter() {
        return mAttributesFilter;
    }

    public void setMAttributesFilter(short mattributesFilter) {
        this.mAttributesFilter = mattributesFilter;
    }

    public boolean isWithScore() {
        return withScore;
    }

    public void setWithScore(boolean withScore) {
        this.withScore = withScore;
    }

    public boolean isWithCorpusName() {
        return withCorpusName;
    }

    public void setWithCorpusName(boolean withCorpusName) {
        this.withCorpusName = withCorpusName;
    }

    public boolean isWithBaseName() {
        return withBaseName;
    }

    public void setWithBaseName(boolean withBaseName) {
        this.withBaseName = withBaseName;
    }

    public boolean withAttributes() {
        return !attributeKeyMap.isEmpty();
    }

    public String getAttributePropertyName(AttributeKey attributeKey) {
        return attributeKeyMap.get(attributeKey);
    }

    public boolean putAttributePropertyName(AttributeKey attributeKey, String name) {
        if (!isValidName(name)) {
            return false;
        }
        attributeKeyMap.put(attributeKey, name);
        return true;
    }

    private boolean isValidName(String name) {
        if (FicheDataJson.isReservedPropertyName(name)) {
            return false;
        }
        if (titrePropertyName != null) {
            if (titrePropertyName.equals(name)) {
                return false;
            }
        }
        if (soustitrePropertyName != null) {
            if (soustitrePropertyName.equals(name)) {
                return false;
            }
        }
        if (hrefPropertyName != null) {
            if (hrefPropertyName.equals(name)) {
                return false;
            }
        }
        if (anneePropertyName != null) {
            if (anneePropertyName.equals(name)) {
                return false;
            }
        }
        if (attributeKeyMap.containsValue(name)) {
            return false;
        }
        return checkPropertyName(name);
    }

    public void buildFromString(String s) {
        String[] tokens = StringUtils.getTechnicalTokens(s, ',', false);
        for (String token : tokens) {
            if (token.equals("codecorpus")) {
                setWithCodeCorpus(true);
            } else if (token.equals("codebase")) {
                setWithCodeBase(true);
            } else if (token.equals("ficheid")) {
                setWithFicheId(true);
            } else if (token.equals("titre")) {
                this.titrePropertyName = "titre";
            } else if (token.equals("soustitre")) {
                this.soustitrePropertyName = "soustitre";
            } else if (token.equals("href")) {
                this.hrefPropertyName = "href";
            } else if (token.equals("lang")) {
                setWithLang(true);
            } else if (token.equals("annee")) {
                this.anneePropertyName = "annee";
            } else if (token.equals("date")) {
                setWithDate(true);
            } else if (token.equals("ficheicon")) {
                setWithFicheIcon(true);
            } else if (token.equals("geo")) {
                setWithGeo(true);
            } else if (token.equals("mtitre") || token.equals("mtitre_def")) {
                setWithMTitre(true);
            } else if (token.equals("msoustitre") || token.equals("msoustitre_def")) {
                setWithMSoustitre(true);
            } else if (token.equals("complements")) {
                setWithComplements(true);
            } else if (token.equals("mcomplements")) {
                setWithMComplements(true);
            } else if (token.equals("mcomplements_all")) {
                setWithMComplements(true);
                setAllMComplements(true);
            } else if (token.equals("score")) {
                setWithScore(true);
            } else if (token.equals("corpusname")) {
                setWithCorpusName(true);
            } else if (token.equals("basename")) {
                setWithBaseName(true);
            } else if (token.equals("icon")) {
                setWithIcon(true);
            } else if (token.equals("mattrs")) {
                setMAttributesFilter(CONCERNED_FILTER);
            } else if (token.equals("mattrs_all")) {
                setMAttributesFilter(ALL_FILTER);
            } else if (token.equals("mattrs_primary")) {
                setMAttributesFilter(PRIMARY_FILTER);
            } else {
                try {
                    AttributeKey attributeKey = AttributeKey.parse(token);
                    attributeKeyMap.put(attributeKey, attributeKey.toString());
                } catch (ParseException pe) {

                }
            }
        }
    }

    public boolean isWithIcon() {
        return withIcon;
    }

    public void setWithIcon(boolean withIcon) {
        this.withIcon = withIcon;
    }

    private static boolean checkPropertyName(String name) {
        int length = name.length();
        for (int i = 0; i < length; i++) {
            char carac = name.charAt(i);
            if (carac == '_') {
                continue;
            } else if ((carac >= 'a') && (carac <= 'z')) {
                continue;
            } else if ((carac >= '0') && (carac <= '9')) {
                continue;
            }
            return false;
        }
        return true;
    }

}
