/* SctServer - Copyright (c) 2010-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import fr.exemole.sctserver.json.JsonUtils;
import java.io.IOException;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.collation.CollationUnit;
import net.scrutari.searchengine.api.result.SupermotcleSearchResult;
import net.scrutari.searchengine.api.result.SupermotcleSearchResultInfo;
import net.scrutari.supermotcle.api.Supermotcle;


/**
 *
 * @author Vincent Calame
 */
public class SupermotcleSearchResultJsonProducer extends AbstractJsonProducer {

    private final int version;
    private final SupermotcleSearchResult supermotcleSearchResult;
    private final Lang langUi;
    private final int limit;

    public SupermotcleSearchResultJsonProducer(int version, SupermotcleSearchResult supermotcleSearchResult, Lang langUi, int limit) {
        this.version = version;
        this.supermotcleSearchResult = supermotcleSearchResult;
        this.langUi = langUi;
        this.limit = limit;
    }

    @Override
    public void writeJson(Appendable appendable) throws IOException {
        JSONWriter jsonWriter = new JSONWriter(appendable);
        try {
            jsonWriter.object();
            jsonWriter.key("supermotcleSearchResult");
            jsonWriter.object();
            int count = supermotcleSearchResult.getSupermotcleSearchResultInfoCount();
            if (limit != -1) {
                count = Math.min(count, limit);
            }
            jsonWriter.key("supermotcleCount");
            jsonWriter.value(count);
            if (count > 0) {
                jsonWriter.key("supermotcleArray");
                jsonWriter.array();
                for (int i = 0; i < count; i++) {
                    SupermotcleSearchResultInfo info = supermotcleSearchResult.getSupermotcleSearchResultInfo(i);
                    Supermotcle supermotcle = info.getSupermotcle();
                    CollationUnit collationUnit = supermotcle.getCollationUnit();
                    String lib = collationUnit.getSourceString();
                    jsonWriter.object();
                    jsonWriter.key("codesupermotcle");
                    jsonWriter.value(supermotcle.getCode());
                    jsonWriter.key("lib");
                    jsonWriter.value(lib);
                    jsonWriter.key("mlib");
                    JsonUtils.writeMLib(jsonWriter, lib, info.getSearchTokentPositionList());
                    int baseCount = supermotcle.getBaseCount();
                    jsonWriter.key("indexationCount");
                    jsonWriter.value(supermotcle.getIndexationCount());
                    jsonWriter.key("baseArray");
                    jsonWriter.array();
                    for (int j = 0; j < baseCount; j++) {
                        jsonWriter.object();
                        jsonWriter.key("codebase");
                        jsonWriter.value(supermotcle.getCodeBase(j));
                        jsonWriter.key("indexationCount");
                        jsonWriter.value(supermotcle.getIndexationCountByBase(j));
                        jsonWriter.endObject();
                    }
                    jsonWriter.endArray();
                    jsonWriter.endObject();
                }
                jsonWriter.endArray();
            }
            jsonWriter.endObject();
            writeWarnings(jsonWriter);
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage(), je);
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(throwable.getMessage(), (Exception) throwable);
                }
            }
        }
    }

}
