/* ScrutariLib_SearchEngine - Copyright (c) 2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.attr.CheckedKeyString;
import net.scrutari.data.BaseData;
import net.scrutari.data.FicheData;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.db.tools.codes.CodesUtils;
import net.scrutari.searchengine.api.options.Category;
import net.scrutari.searchengine.api.options.GlobalSearchOptions;
import net.scrutari.searchengine.tools.options.SearchOptionsBuilder;
import net.scrutari.searchengine.tools.options.SearchOptionsDefBuilder;


/**
 *
 * @author Vincent Calame
 */
public final class BuildingUtils {

    public final static CheckedKeyString CATEGORY_NAMESPACE = CheckedKeyString.build("category");
    public final static AttributeKey CATEGORY_LIST_KEY = AttributeKey.build(CATEGORY_NAMESPACE, "list");

    private BuildingUtils() {

    }

    public static void computeReduction(SearchOptionsBuilder searchOptionsBuilder, DataAccess dataAccess, GlobalSearchOptions globalSearchOptions) {
        SearchOptionsDefBuilder searchOptionsDefBuilder = searchOptionsBuilder.getSearchOptionsDefBuilder();
        Set<ScrutariDataURI> ficheURISet = searchOptionsDefBuilder.getURISet(ScrutariDataURI.FICHEURI_TYPE);
        if (!ficheURISet.isEmpty()) {
            Map<Integer, Integer> elementMap = new LinkedHashMap<Integer, Integer>();
            for (ScrutariDataURI scrutariDataURI : ficheURISet) {
                int code = dataAccess.getCode(scrutariDataURI);
                if (code > 0) {
                    if (!elementMap.containsKey(code)) {
                        FicheData ficheData = dataAccess.getFicheDataByCode(code);
                        elementMap.put(ficheData.getCodeFiche(), ficheData.getCodeCorpus());
                    }
                }
            }
            searchOptionsBuilder.reduceFicheList(CodesUtils.toElementCodeList(elementMap));
        }
        Set<ScrutariDataURI> baseURISet = searchOptionsDefBuilder.getURISet(ScrutariDataURI.BASEURI_TYPE);
        Set<ScrutariDataURI> corpusSURISet = searchOptionsDefBuilder.getURISet(ScrutariDataURI.CORPUSURI_TYPE);
        if ((!baseURISet.isEmpty()) || (!corpusSURISet.isEmpty())) {
            Set<Integer> codeSet = new LinkedHashSet<Integer>();
            for (ScrutariDataURI scrutariDataURI : corpusSURISet) {
                int code = dataAccess.getCode(scrutariDataURI);
                if (code > 0) {
                    codeSet.add(code);
                }
            }
            for (ScrutariDataURI scrutariDataURI : baseURISet) {
                int code = dataAccess.getCode(scrutariDataURI);
                if (code > 0) {
                    BaseData baseData = dataAccess.getBaseDataByCode(code);
                    int corpusCount = baseData.getCorpusCount();
                    for (int j = 0; j < corpusCount; j++) {
                        codeSet.add(baseData.getCodeCorpus(j));
                    }
                }
            }
            searchOptionsBuilder.reduceCorpusList(CodesUtils.toCodeList(codeSet));
        }
        Attribute attribute = searchOptionsDefBuilder.getAttribute(CATEGORY_LIST_KEY);
        if (attribute != null) {
            int count = attribute.getValueCount();
            List<Integer> selectionList = new ArrayList<Integer>();
            for (int i = 0; i < count; i++) {
                String categoryName = attribute.getValue(i);
                Category category = globalSearchOptions.getCategoryByName(categoryName);
                if (category != null) {
                    CodeList codeList = category.getCorpusCodeList();
                    int codeCount = codeList.getCodeCount();
                    for (int j = 0; j < codeCount; j++) {
                        selectionList.add(codeList.getCode(j));
                    }
                }
            }
            searchOptionsBuilder.reduceCorpusList(CodesUtils.toCodeList(selectionList));
        }
        Set<ScrutariDataURI> thesaurusURISet = searchOptionsDefBuilder.getURISet(ScrutariDataURI.THESAURUSURI_TYPE);
        if (!thesaurusURISet.isEmpty()) {
            Set<Integer> codeSet = new LinkedHashSet<Integer>();
            for (ScrutariDataURI scrutariDataURI : thesaurusURISet) {
                int code = dataAccess.getCode(scrutariDataURI);
                if (code > 0) {
                    if (!codeSet.contains(code)) {
                        codeSet.add(code);
                    }
                }
            }
            searchOptionsBuilder.reduceThesaurusList(CodesUtils.toCodeList(codeSet));
        }
    }
}
