/* SctServer - Copyright (c) 2012-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.conf.dom;

import fr.exemole.sctserver.tools.ErrorUtils;
import java.text.ParseException;
import net.mapeadores.util.attr.AttributeDef;
import net.mapeadores.util.attr.AttributeDefBuilder;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelUtils;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.ElementHandler;
import net.scrutari.data.ScrutariAttributes;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class AttributesDOMReader {

    private final Builder builder;
    private final MessageHandler messageHandler;

    public AttributesDOMReader(Builder builder, MessageHandler messageHandler) {
        this.builder = builder;
        this.messageHandler = messageHandler;
    }

    public void readAttributes(Element element_xml, String xpath) {
        xpath = xpath + "/" + element_xml.getTagName();
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("group")) {
                    readGroup(element, xpath);
                } else {
                    ErrorUtils.warning(messageHandler, "_ error.unknown.xml.tag", xpath + "/" + tagName);
                }
            }
        }
    }

    private void readGroup(Element element_xml, String xpath) {
        xpath = xpath + "/" + element_xml.getTagName();
        String name = element_xml.getAttribute("name");
        if (name.length() == 0) {
            ErrorUtils.severe(messageHandler, "_ error.empty.xml.attribute", xpath, "name");
            return;
        }
        xpath = xpath + "[@name='" + name + "']";
        boolean start = builder.startGroup(name);
        if (!start) {
            ErrorUtils.warning(messageHandler, "_ error.unknown.xml.attributevalue", xpath, "name", name);
            return;
        }
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("attr-def")) {
                    String localKey = element.getAttribute("key").trim();
                    if (localKey.length() == 0) {
                        ErrorUtils.severe(messageHandler, "_ error.empty.xml.attribute", xpath + "/" + tagName, "key");
                        continue;
                    }
                    String nameSpace = element.getAttribute("ns").trim();
                    if (nameSpace.length() == 0) {
                        ErrorUtils.severe(messageHandler, "_ error.empty.xml.attribute", xpath + "/" + tagName, "ns");
                        continue;
                    }
                    String xpath2 = xpath + "/" + tagName + "[@ns='" + nameSpace + "' and @key='" + localKey + "']";
                    AttributeKey attributeKey;
                    try {
                        attributeKey = ScrutariAttributes.toAttributeKey(nameSpace, localKey);
                    } catch (ParseException pe) {
                        ErrorUtils.severe(messageHandler, "_ error.wrong.xml.attributevalue", xpath2, "key|ns", nameSpace + ":" + localKey);
                        continue;
                    }
                    AttributeDefHandler attributeDefHandler = new AttributeDefHandler(attributeKey, xpath2);
                    DOMUtils.readChildren(element, attributeDefHandler);
                    boolean done = builder.addAttributeDef(attributeDefHandler.toAttributeDef());
                    if (!done) {
                        ErrorUtils.warning(messageHandler, "_ error.existing.xml.attributevalue", xpath2, "key|ns", attributeKey.toString());
                    }
                } else {
                    ErrorUtils.warning(messageHandler, "_ error.unknown.xml.tag", xpath + "/" + tagName);
                }
            }
        }
    }


    private class AttributeDefHandler implements ElementHandler {

        private final AttributeKey attributeKey;
        private final String xpath;
        private final AttributeDefBuilder builder;

        private AttributeDefHandler(AttributeKey attributeKey, String xpath) {
            this.attributeKey = attributeKey;
            this.xpath = xpath;
            AttributeDef defaultAttributeKey = ScrutariAttributes.getDefaultAttributeDef(attributeKey);
            if (defaultAttributeKey != null) {
                builder = AttributeDefBuilder.derive(defaultAttributeKey);
            } else {
                builder = new AttributeDefBuilder(attributeKey);
            }
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            if ((tagName.equals("label")) || (tagName.equals("lib"))) {
                try {
                    Label libelle = LabelUtils.readLabel(element);
                    if (libelle != null) {
                        builder.addLabel(libelle);
                    }
                } catch (ParseException pe) {
                    ErrorUtils.severe(messageHandler, "_ error.wrong.xml.langattributevalue", xpath + tagName, element.getAttribute("xml:lang"));
                }
            } else if (tagName.equals("format")) {
                String type = element.getAttribute("type");
                if (ScrutariAttributes.isValidFormatType(type)) {
                    builder.setFormatType(type);
                } else {
                    ErrorUtils.warning(messageHandler, "_ error.unknown.xml.attributevalue", xpath + tagName, "type", type);
                }
            }
        }

        public AttributeDef toAttributeDef() {
            return builder.toAttributeDef();
        }

    }


    public static interface Builder {

        public boolean startGroup(String name);

        public boolean addAttributeDef(AttributeDef attributeDef);

    }

}
