/* SctServer - Copyright (c) 2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.conf.dom;

import fr.exemole.sctserver.tools.ErrorUtils;
import net.mapeadores.util.logging.MessageHandler;
import net.scrutari.searchengine.api.options.PertinencePonderation;
import net.scrutari.searchengine.tools.options.PertinencePonderationBuilder;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class OptionsDOMReader {

    private final Builder builder;
    private final MessageHandler messageHandler;

    public OptionsDOMReader(Builder builder, MessageHandler messageHandler) {
        this.builder = builder;
        this.messageHandler = messageHandler;
    }

    public void readOptions(Element element_xml) {
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("ponderation")) {
                    setPonderation(element);
                } else {
                    ErrorUtils.warning(messageHandler, "_ error.unknown.xml.tag", tagName);
                }
            }
        }
    }

    private void setPonderation(Element element_xml) {
        int occurrence = getInt(element_xml, "occurrence");
        int date = getInt(element_xml, "date");
        int origin = getInt(element_xml, "origin");
        builder.setGlobalPertinencePonderation(PertinencePonderationBuilder.toPertinencePonderation(occurrence, date, origin));
    }

    private int getInt(Element element_xml, String attribute) {
        String vale = element_xml.getAttribute(attribute);
        if (vale.length() == 0) {
            return 0;
        }
        try {
            int itg = Integer.parseInt(vale);
            return itg;
        } catch (NumberFormatException nfe) {
            return 0;
        }
    }


    public static interface Builder {

        public void setGlobalPertinencePonderation(PertinencePonderation pertinencePonderation);

    }

}
