/* SctServer - Copyright (c) 2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.xml.feed;

import fr.exemole.sctserver.api.ScrutariSession;
import fr.exemole.sctserver.api.feed.SctFeedConstants;
import fr.exemole.sctserver.api.feed.SctFeedInfo;
import fr.exemole.sctserver.api.feed.SctFeedOptions;
import java.io.IOException;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.xml.AppendableXMLPart;
import net.mapeadores.util.xml.AppendableXMLWriter;
import net.scrutari.db.api.DataAccess;


/**
 *
 * @author Vincent Calame
 */
public abstract class AbstractEntriesXMLPart extends AppendableXMLPart {

    protected final ScrutariSession scrutariSession;

    public AbstractEntriesXMLPart(AppendableXMLWriter xmlWriter, ScrutariSession scrutariSession) {
        super(xmlWriter);
        this.scrutariSession = scrutariSession;
    }

    public abstract void appendEntries(String feedId, SctFeedInfo sctFeedInfo, DataAccess dataAccess, SctFeedOptions sctFeedOptions) throws IOException;

    static AbstractEntriesXMLPart newInstance(short type, AppendableXMLWriter xmlWriter, ScrutariSession scrutariSession) {
        switch (type) {
            case SctFeedConstants.TREE_SCTFEED_TYPE:
                return new TreeEntriesXMLPart(xmlWriter, scrutariSession);
            case SctFeedConstants.CATEGORIES_SCTFEED_TYPE:
                return new CategoriesEntriesXMLPart(xmlWriter, scrutariSession);
            case SctFeedConstants.FICHES_SCTFEED_TYPE:
                return new FichesEntriesXMLPart(xmlWriter, scrutariSession);
            default:
                throw new SwitchException("type=" + type);
        }
    }

}
