/* UtilLib - Copyright (c) 2011-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.format;


/**
 *
 * @author Vincent Calame
 */
public final class FormatUtils {

    private FormatUtils() {
    }

    public static short toCastType(Object value) {
        if (value == null) {
            return FormatConstants.NO_CAST;
        }
        try {
            return ((Short) value);
        } catch (ClassCastException cce) {
            return FormatConstants.NO_CAST;
        }
    }

    public static boolean toBoolean(Object value) {
        return toBoolean(value, false);
    }

    public static boolean toBoolean(Object value, boolean defaultValue) {
        if (value == null) {
            return defaultValue;
        }
        try {
            return ((Boolean) value);
        } catch (ClassCastException cce) {
            return defaultValue;
        }
    }

    public static Calcul toCalcul(Object value) {
        if (value == null) {
            return null;
        }
        try {
            return (Calcul) value;
        } catch (ClassCastException cce) {
            return null;
        }
    }

    public static int toPositiveInteger(Object value) {
        if (value == null) {
            return -1;
        }
        try {
            int itg = (Integer) value;
            if (itg < 0) {
                return -1;
            }
            return itg;
        } catch (ClassCastException cce) {
            return -1;
        }
    }

    public static String toString(Object value) {
        if (value == null) {
            return null;
        }
        return value.toString();
    }

    public static char toChar(Object value) {
        if (value == null) {
            return ' ';
        }
        try {
            return ((Character) value);
        } catch (ClassCastException cce) {
            return ' ';
        }
    }

    public static String getSeparator(FormatDef formatDef, String defaultSeparator) {
        if (isWithSum(formatDef)) {
            return ";";
        }
        Object value = formatDef.getParameterValue(FormatConstants.SEPARATOR_PARAMKEY);
        if (value == null) {
            return defaultSeparator;
        }
        return (String) value;
    }

    public static String getInnerSeparator(FormatDef formatDef, int sourceIndex, String defaultSeparator) {
        if (isWithSum(formatDef)) {
            return ";";
        }
        FormatDef.InnerSeparator sep = getInnerSeparatorObject(formatDef, sourceIndex);
        if (sep == null) {
            return getSeparator(formatDef, defaultSeparator);
        } else {
            return sep.getSeparator();
        }
    }

    private static boolean isWithSum(FormatDef formatDef) {
        return (formatDef.getParameterValue(FormatConstants.SUM_PARAMKEY) != null);
    }

    private static FormatDef.InnerSeparator getInnerSeparatorObject(FormatDef formatDef, int sourceIndex) {
        int count = formatDef.getInnerSeparatorCount();
        if (count == 0) {
            return null;
        }
        for (int i = 0; i < count; i++) {
            FormatDef.InnerSeparator innerSeparator = formatDef.getInnerSeparator(i);
            if (innerSeparator.getSourceIndex() == sourceIndex) {
                return innerSeparator;
            }
        }
        return null;
    }

    public static String getSourceSeparator(FormatDef formatDef, int sourceIndex1, int sourceIndex2, String defaultSeparator) {
        if (isWithSum(formatDef)) {
            return ";";
        }
        FormatDef.SourceSeparator sep = getSourceSeparatorObject(formatDef, sourceIndex1, sourceIndex2);
        if (sep == null) {
            return getSeparator(formatDef, defaultSeparator);
        } else {
            return sep.getSeparator();
        }
    }

    private static FormatDef.SourceSeparator getSourceSeparatorObject(FormatDef formatDef, int sourceIndex1, int sourceIndex2) {
        int count = formatDef.getSourceSeparatorCount();
        if (count == 0) {
            return null;
        }
        if (sourceIndex1 > sourceIndex2) {
            int tmp = sourceIndex1;
            sourceIndex1 = sourceIndex2;
            sourceIndex2 = tmp;
        }
        FormatDef.SourceSeparator nearestSourceSeparator = null;
        for (int i = 0; i < count; i++) {
            FormatDef.SourceSeparator sourceSeparator = formatDef.getSourceSeparator(i);
            if (sourceSeparator.getSourceIndex1() == sourceIndex1) {
                int si2 = sourceSeparator.getSourceIndex2();
                if (si2 == sourceIndex2) {
                    return sourceSeparator;
                } else if (si2 > sourceIndex2) {
                    continue;
                } else {
                    if ((nearestSourceSeparator == null) || (nearestSourceSeparator.getSourceIndex2() < si2)) {
                        nearestSourceSeparator = sourceSeparator;
                    }
                }
            }
        }
        if (nearestSourceSeparator != null) {
            return nearestSourceSeparator;
        }
        return null;
    }

    public static short castTypeToShort(String s) {
        switch (s) {
            case FormatConstants.INTEGER_PARAMVALUE:
            case "int":
                return FormatConstants.INTEGER_CAST;
            case FormatConstants.DECIMAL_PARAMVALUE:
            case "double":
            case "float":
                return FormatConstants.DECIMAL_CAST;
            case FormatConstants.DATE_PARAMVALUE:
                return FormatConstants.DATE_CAST;
            case FormatConstants.MONEY_PARAMVALUE:
                return FormatConstants.MONEY_CAST;
            case FormatConstants.PERCENTAGE_PARAMVALUE:
                return FormatConstants.PERCENTAGE_CAST;
            default:
                throw new IllegalArgumentException("Unknown cast type : " + s);
        }
    }

    public static String castTypeToString(short castType) {
        switch (castType) {
            case FormatConstants.INTEGER_CAST:
                return FormatConstants.INTEGER_PARAMVALUE;
            case FormatConstants.DECIMAL_CAST:
                return FormatConstants.DECIMAL_PARAMVALUE;
            case FormatConstants.DATE_CAST:
                return FormatConstants.DATE_PARAMVALUE;
            case FormatConstants.MONEY_CAST:
                return FormatConstants.MONEY_PARAMVALUE;
            case FormatConstants.PERCENTAGE_CAST:
                return FormatConstants.PERCENTAGE_PARAMVALUE;
            default:
                throw new IllegalArgumentException("Unknown cast type : " + castType);
        }
    }

}
