/* UtilLib - Copyright (c) 2006-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.html;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;


/**
 *
 * @author Vincent Calame
 */
public class HTMLAttributes implements Serializable, Cloneable {

    private String name;
    private String classes;
    private String id;
    private String style;
    private String target;
    private String href;
    private String src;
    private String alt;
    private String title;
    private String width;
    private String height;
    private String size;
    private String value;
    private String type;
    private String action;
    private String method;
    private String label;
    private String rel;
    private String content;
    private String httpEquiv;
    private String scrolling;
    private String enctype;
    private String span;
    private boolean checked;
    private boolean selected;
    private boolean disabled;
    private boolean readonly;
    private boolean multiple;
    private boolean required;
    private int colSpan;
    private int rowSpan;
    private int cols;
    private int rows;
    private Map<String, String> attrMap = null;

    public HTMLAttributes() {
    }

    public HTMLAttributes copy() {
        return (HTMLAttributes) clone();
    }

    @Override
    public Object clone() {
        try {
            return super.clone();
        } catch (CloneNotSupportedException e) {
            return null;
        }
    }

    public String classes() {
        return classes;
    }

    public HTMLAttributes classes(String className) {
        this.classes = className;
        return this;
    }

    public HTMLAttributes addClass(String className) {
        if ((className == null) || (className.length() == 0)) {
            return this;
        }
        if ((classes == null) || (classes.length() == 0)) {
            classes = className;
        } else {
            classes = classes + " " + className;
        }
        return this;
    }

    public String id() {
        return id;
    }

    public HTMLAttributes id(String id) {
        this.id = id;
        return this;
    }

    public String name() {
        return name;
    }

    public HTMLAttributes name(String name) {
        this.name = name;
        return this;
    }

    public String style() {
        return style;
    }

    public HTMLAttributes style(String style) {
        this.style = style;
        return this;
    }

    public String target() {
        return target;
    }

    public HTMLAttributes target(String target) {
        this.target = target;
        return this;
    }

    public String href() {
        return href;
    }

    public HTMLAttributes href(String href) {
        this.href = href;
        return this;
    }

    public HTMLAttributes href(HrefBuilder hrefBuilder) {
        this.href = hrefBuilder.toHref();
        return this;
    }

    public String width() {
        return width;
    }

    public HTMLAttributes width(String width) {
        this.width = width;
        return this;
    }

    public String height() {
        return height;
    }

    public HTMLAttributes height(String height) {
        this.height = height;
        return this;
    }

    public String title() {
        return title;
    }

    public HTMLAttributes title(String title) {
        this.title = title;
        return this;
    }

    public String src() {
        return src;
    }

    public HTMLAttributes src(String src) {
        this.src = src;
        return this;
    }

    public String alt() {
        return alt;
    }

    public HTMLAttributes alt(String alt) {
        this.alt = alt;
        return this;
    }

    public String size() {
        return size;
    }

    public HTMLAttributes size(String size) {
        this.size = size;
        return this;
    }

    public HTMLAttributes value(String value) {
        this.value = value;
        return this;
    }

    public String value() {
        return value;
    }

    public HTMLAttributes type(String type) {
        this.type = type;
        return this;
    }

    public String type() {
        return type;
    }

    public boolean checked() {
        return checked;
    }

    public HTMLAttributes checked(boolean checked) {
        this.checked = checked;
        return this;
    }

    public boolean selected() {
        return selected;
    }

    public HTMLAttributes selected(boolean selected) {
        this.selected = selected;
        return this;
    }

    public boolean disabled() {
        return disabled;
    }

    public HTMLAttributes disabled(boolean disabled) {
        this.disabled = disabled;
        return this;
    }

    public boolean multiple() {
        return multiple;
    }

    public HTMLAttributes multiple(boolean multiple) {
        this.multiple = multiple;
        return this;
    }

    public String action() {
        return action;
    }

    public HTMLAttributes action(String action) {
        this.action = action;
        return this;
    }

    public String method() {
        return method;
    }

    public HTMLAttributes method(String method) {
        this.method = method;
        return this;
    }

    public int colspan() {
        return colSpan;
    }

    public HTMLAttributes colspan(int colSpan) {
        this.colSpan = colSpan;
        return this;
    }

    public int rowspan() {
        return rowSpan;
    }

    public HTMLAttributes rowspan(int rowSpan) {
        this.rowSpan = rowSpan;
        return this;
    }

    public String label() {
        return label;
    }

    public HTMLAttributes label(String label) {
        this.label = label;
        return this;
    }

    public int cols() {
        return cols;
    }

    public HTMLAttributes cols(int cols) {
        this.cols = cols;
        return this;
    }

    public int rows() {
        return rows;
    }

    public HTMLAttributes rows(int rows) {
        this.rows = rows;
        return this;
    }

    public boolean readonly() {
        return readonly;
    }

    public HTMLAttributes readonly(boolean readonly) {
        this.readonly = readonly;
        return this;
    }

    public String rel() {
        return rel;
    }

    public HTMLAttributes rel(String rel) {
        this.rel = rel;
        return this;
    }

    public String content() {
        return content;
    }

    public HTMLAttributes content(String content) {
        this.content = content;
        return this;
    }

    public String httpEquiv() {
        return httpEquiv;
    }

    public HTMLAttributes httpEquiv(String httpEquiv) {
        this.httpEquiv = httpEquiv;
        return this;
    }

    public String scrolling() {
        return scrolling;
    }

    public HTMLAttributes scrolling(String scrolling) {
        this.scrolling = scrolling;
        return this;
    }

    public String enctype() {
        return enctype;
    }

    public HTMLAttributes enctype(String enctype) {
        this.enctype = enctype;
        return this;
    }

    public String span() {
        return span;
    }

    public HTMLAttributes span(String span) {
        this.span = span;
        return this;
    }

    public boolean required() {
        return required;
    }

    public HTMLAttributes required(boolean required) {
        this.required = required;
        return this;
    }

    public HTMLAttributes attr(String name, String value) {
        if (attrMap == null) {
            attrMap = new HashMap<String, String>();
        }
        attrMap.put(name, value);
        return this;
    }

    public String attr(String name) {
        if (attrMap == null) {
            return null;
        }
        return attrMap.get(name);
    }

    public Map<String, String> getAttrMap() {
        return attrMap;
    }

}
