/* UtilLib - Copyright (c) 2010-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.logging;

import java.io.IOException;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.localisation.Message;
import net.mapeadores.util.localisation.MessageList;
import net.mapeadores.util.localisation.MessageLocalisation;


/**
 *
 * @author Vincent Calame
 */
public class CommandMessageJsonProducer implements JsonProducer {

    private CommandMessage commandMessage;
    private MessageLocalisation messageLocalisation;
    private boolean errorMessage = false;
    private String messageKey;
    private String messageText;

    public CommandMessageJsonProducer(CommandMessage commandMessage, MessageLocalisation messageLocalisation) {
        if (commandMessage == null) {
            throw new IllegalArgumentException("commandMessage is null");
        }
        this.errorMessage = commandMessage.isErrorMessage();
        this.messageKey = commandMessage.getMessageKey();
        this.commandMessage = commandMessage;
        this.messageLocalisation = messageLocalisation;
    }

    public CommandMessageJsonProducer(String messageKey, String messageText, boolean errorMessage) {
        this.errorMessage = errorMessage;
        this.messageKey = messageKey;
        this.messageText = messageText;
    }

    @Override
    public void writeJson(Appendable appendable) throws IOException {
        JSONWriter jsonWriter = new JSONWriter(appendable);
        try {
            jsonWriter.object();
            jsonWriter.key("commandMessage");
            jsonWriter.object();
            writeType(jsonWriter);
            writeMessage(jsonWriter);
            jsonWriter.endObject();
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage());
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(je.getMessage(), (Exception) throwable);
                }
            }
        }
    }

    private void writeType(JSONWriter jsonWriter) throws JSONException {
        jsonWriter.key("type");
        if (errorMessage) {
            jsonWriter.value("error");
        } else {
            jsonWriter.value("done");
        }
    }

    private void writeMessage(JSONWriter jsonWriter) throws JSONException {
        jsonWriter.key("key");
        jsonWriter.value(messageKey);
        if (messageText != null) {
            jsonWriter.key("text");
            jsonWriter.value(messageText);
        } else if (commandMessage != null) {
            if (messageLocalisation != null) {
                jsonWriter.key("text");
                jsonWriter.value(messageLocalisation.toString(commandMessage));
            }
            if (commandMessage.hasMultiError()) {
                MessageList messageList = commandMessage.getMultiErrorList();
                int count = messageList.getMessageCount();
                jsonWriter.key("array");
                jsonWriter.array();
                for (int i = 0; i < count; i++) {
                    Message multiErrorMessage = messageList.getMessage(i);
                    jsonWriter.object();
                    jsonWriter.key("key");
                    jsonWriter.value(multiErrorMessage.getMessageKey());
                    if (messageLocalisation != null) {
                        jsonWriter.key("text");
                        jsonWriter.value(messageLocalisation.toString(multiErrorMessage));
                    }
                    jsonWriter.endObject();
                }
                jsonWriter.endArray();
            }
        }
    }

}
