/* UtilLib - Copyright (c) 2013-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.logging;

import java.util.ArrayList;
import java.util.List;
import net.mapeadores.util.localisation.Message;


/**
 *
 * @author Vincent Calame
 */
public class MessageLogBuilder implements MultiMessageHandler {

    private final CategorizedMessageListBuilder currentBuilder = new CategorizedMessageListBuilder();
    private String currentURI = null;
    private final List<InternalLogGroup> groupList = new ArrayList<InternalLogGroup>();

    public MessageLogBuilder() {
    }

    @Override
    public void setCurrentURI(String uri) {
        if (currentBuilder.isEmpty()) {
            currentURI = uri;
        } else {
            boolean flush = false;
            if (uri == null) {
                if (currentURI != null) {
                    flush = true;
                }
            } else if (currentURI == null) {
                flush = true;
            } else {
                flush = !uri.equals(currentURI);
            }
            if (flush) {
                flushGroup();
                currentURI = uri;
            }
        }
    }

    @Override
    public void addMessage(String category, Message message) {
        currentBuilder.add(category, message);
    }

    private void flushGroup() {
        if (!currentBuilder.isEmpty()) {
            groupList.add(new InternalLogGroup(currentURI, currentBuilder.toCategorizedMessageList()));
            currentBuilder.clear();
        }
    }

    public MessageLog toMessageLog() {
        flushGroup();
        return new InternalMessageLog(groupList.toArray(new MessageLog.LogGroup[groupList.size()]));
    }


    private static class InternalLogGroup implements MessageLog.LogGroup {

        private final String processURI;
        private final CategorizedMessageList messageList;

        private InternalLogGroup(String processURI, CategorizedMessageList messageList) {
            this.processURI = processURI;
            this.messageList = messageList;
        }

        @Override
        public String getURI() {
            return processURI;
        }

        @Override
        public CategorizedMessageList getList() {
            return messageList;
        }

    }


    private static class InternalMessageLog implements MessageLog {

        private final MessageLog.LogGroup[] array;

        private InternalMessageLog(MessageLog.LogGroup[] array) {
            this.array = array;
        }

        @Override
        public int getLogGroupCount() {
            return array.length;
        }

        @Override
        public LogGroup getLogGroup(int i) {
            return array[i];
        }

    }

}
