/* UtilLib - Copyright (c) 2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.primitives;

import java.io.IOException;
import java.util.Arrays;
import javax.xml.bind.DatatypeConverter;
import net.mapeadores.util.primitives.io.PrimitivesReader;
import net.mapeadores.util.primitives.io.PrimitivesWriter;


/**
 *
 * @author Vincent Calame
 */
public final class ByteArrayKey {

    private final byte[] byteArray;
    private final int hashCode;

    public ByteArrayKey(byte[] byteArray) {
        if (byteArray == null) {
            throw new NullPointerException();
        }
        this.byteArray = byteArray;
        this.hashCode = Arrays.hashCode(byteArray);
    }

    @Override
    public boolean equals(Object other) {
        if (other == null) {
            return false;
        }
        if (other == this) {
            return true;
        }
        if (other.getClass() != this.getClass()) {
            return false;
        }
        ByteArrayKey otherByteArrayKey = (ByteArrayKey) other;
        if (otherByteArrayKey.hashCode != this.hashCode) {
            return false;
        }
        return Arrays.equals(otherByteArrayKey.byteArray, this.byteArray);
    }

    @Override
    public int hashCode() {
        return hashCode;
    }

    @Override
    public String toString() {
        return DatatypeConverter.printBase64Binary(byteArray);
    }

    public static ByteArrayKey fromPrimitives(PrimitivesReader primitivesReader) throws IOException {
        int length = primitivesReader.readInt();
        byte[] array = primitivesReader.readByteArray(length);
        return new ByteArrayKey(array);
    }

    public static void toPrimitives(PrimitivesWriter primitivesWriter, ByteArrayKey byteArrayKey) throws IOException {
        int length = byteArrayKey.byteArray.length;
        primitivesWriter.writeInt(length);
        primitivesWriter.writeByteArray(byteArrayKey.byteArray);
    }

}
